@@ -2,15 +2,27 @@
 # License: BSD
 # Copyright: (C) Dmitry Shachnev, 2012
 
-# MathJax pattern code is based on work by Rob Mayoff
-# https://github.com/mayoff/python-markdown-mathjax
-
 from __future__ import absolute_import
 
 import sys
 from markups.core import *
 from markups.abstract import AbstractMarkup
 
+MATHJAX_CONFIG = '''
+<script type="text/x-mathjax-config">
+MathJax.Hub.Config({
+  tex2jax: {
+    inlineMath: [ ["$", "$"], ["\\\\(", "\\\\)"] ],
+    processEscapes: true,
+    processClass: "math"
+  },
+  TeX: {
+    equationNumbers: {autoNumber: "AMS"}
+  }
+});
+</script>
+'''
+
 class MarkdownMarkup(AbstractMarkup):
 	"""Markdown language"""
 	name = 'Markdown'
@@ -52,14 +64,36 @@ class MarkdownMarkup(AbstractMarkup):
 				return False
 		return True
 	
-	def _get_mathjax_pattern(self, markdown):
-		def handleMatch(m):
-			node = markdown.util.etree.Element('mathjax')
-			node.text = markdown.util.AtomicString(m.group(2) + m.group(3) + m.group(2))
+	def _get_mathjax_patterns(self, markdown):
+		def handle_match_inline(m):
+			node = markdown.util.etree.Element('span')
+			node.set('class', 'math')
+			node.text = markdown.util.AtomicString(m.group(2) + m.group(3) + m.group(4))
 			return node
-		pattern = markdown.inlinepatterns.Pattern(r'(?<!\\)(\$\$?)(.+?)\2')
-		pattern.handleMatch = handleMatch
-		return pattern
+		
+		def handle_match(m):
+			node = markdown.util.etree.Element('div')
+			node.set('class', 'math')
+			node.text = markdown.util.AtomicString(m.group(2) + m.group(3) + m.group(4))
+			return node
+		
+		inlinemathpatterns = (
+			markdown.inlinepatterns.Pattern(r'(?<!\\|\$)(\$)([^\$]+)(\$)'),
+			markdown.inlinepatterns.Pattern(r'(?<!\\)(\\\()([^\\)]+)(\\\))')
+		)
+		mathpatterns = (
+			markdown.inlinepatterns.Pattern(r'(?<!\\)(\$\$)([^\$]+)(\$\$)'),
+			markdown.inlinepatterns.Pattern(r'(?<!\\)(\\\[)(.+)(\\\])'),
+			markdown.inlinepatterns.Pattern(r'(?<!\\)(\\begin{[a-z]+\*?})(.+)(\\end{[a-z]+\*?})')
+		)
+		patterns = []
+		for pattern in inlinemathpatterns:
+			pattern.handleMatch = handle_match_inline
+			patterns.append(pattern)
+		for pattern in mathpatterns:
+			pattern.handleMatch = handle_match
+			patterns.append(pattern)
+		return patterns
 	
 	def __init__(self, filename=None):
 		AbstractMarkup.__init__(self, filename)
@@ -86,15 +120,20 @@ class MarkdownMarkup(AbstractMarkup):
 				self.extensions.remove(extension)
 		self.md = markdown.Markdown(self.extensions, output_format='html4')
 		if self.mathjax:
-			self.md.inlinePatterns.add('mathjax',
-				self._get_mathjax_pattern(markdown), '<escape')
+			self.body_tag = 'class="tex2jax_ignore"'
+			patterns = self._get_mathjax_patterns(markdown)
+			for i in range(len(patterns)):
+				self.md.inlinePatterns.add('mathjax%d' % i, patterns[i], '<escape')
 	
 	def get_document_title(self, text):
 		if 'meta' not in self.extensions:
 			return ''
 		if not 'body' in self.cache:
 			self.get_document_body(text)
-		return str.join(' ', self.md.Meta['title'])
+		if 'title' in self.md.Meta:
+			return str.join(' ', self.md.Meta['title'])
+		else:
+			return ''
 	
 	def get_stylesheet(self, text=''):
 		if 'codehilite' in self.extensions:
@@ -108,16 +147,14 @@ class MarkdownMarkup(AbstractMarkup):
 			body = self.cache['body']
 		else:
 			body = self.get_document_body(text)
-		if not '<math' in body:
+		if not 'class="math"' in body:
 			return ''
 		return ('<script type="text/javascript" src="' + get_mathjax_url(webenv)
-		+ '?config=TeX-AMS-MML_HTMLorMML"></script>\n<script type="text/javascript">\n' +
-		'MathJax.Hub.Config({ "tex2jax": { inlineMath: [[ \'$\', \'$\' ]] } });\n'
-		+ '</script>\n')
+		+ '?config=TeX-AMS-MML_HTMLorMML"></script>' + MATHJAX_CONFIG)
 	
 	def get_document_body(self, text):
 		self.md.reset()
-		converted_text = self.md.convert(text)
+		converted_text = self.md.convert(text) + '\n'
 		if self.enable_cache:
 			self.cache['body'] = converted_text
 		return converted_text
